if Gem.loaded_specs.keys.include?('core_models')

  Rails.application.config.to_prepare do

    # Code extensions
    module CoreModels
      Base::Address.class_eval do
        def format_for_envelope
          output = ''
          output += "#{room}\n" unless room.blank?
          output += "#{building}\n" unless building.blank?
          output += "#{address_line_1}\n" unless address_line_1.blank?
          output += "#{address_line_2}\n" unless address_line_2.blank?
          output += "#{town}\n" unless town.blank?
          output += "#{city}\n" unless city.blank?
          output += "#{region}\n" unless region.blank?
          output += "#{postal_code}\n" unless postal_code.blank?
          output += "#{country.best_name.humanize}" unless country.blank?
          output
        end

        def format_for_invoice
          output = ''
          output += "#{room}\n" unless room.blank?
          output += "#{building}\n" unless building.blank?
          output += "#{address_line_1}\n" unless address_line_1.blank?
          output += "#{address_line_2}\n" unless address_line_2.blank?
          output += "#{town}\n" unless town.blank?
          output += "#{city}\n" unless city.blank?
          output += "#{region}\n" unless region.blank?
          output += "#{postal_code}\n" unless postal_code.blank?
          output += "#{country.best_name.humanize}" unless country.blank?
          output
        end
      end

      #----------------------------------------------------------------------------

      PeopleAndCompanies::Person.class_eval do
        has_many :accounts, :as => :owner, :class_name => '::AccountsManagement::Ledgers::Account'
        has_many :contracts, :as => :first_party, :class_name => '::AccountsManagement::Documents::Contract'
        has_many :contracts, :as => :second_party, :class_name => '::AccountsManagement::Documents::Contract'
        has_many :bank_accounts, :as => :account_owner, :class_name => '::AccountsManagement::Accounts::BankAccount'

        def address_for_invoice
          return postal_address.format_for_invoice if postal_address
          return contact_address.format_for_invoice if contact_address
          return home_address.format_for_invoice if home_address
          return email_1 if email_1
          return email_2 if email_2
          return email_3 if email_3
          nil
        end

        def vat_number_for_invoice
          'n/a'
        end
      end

      #----------------------------------------------------------------------------

      PeopleAndCompanies::Company.class_eval do
        has_many :accounts, :as => :owner, :class_name => '::AccountsManagement::Ledgers::Account'
        has_many :contracts_as_first_party, :as => :first_party, :class_name => '::AccountsManagement::Documents::Contract'
        has_many :contracts_as_second_party, :as => :second_party, :class_name => '::AccountsManagement::Documents::Contract'
        has_many :bank_accounts, :as => :account_owner, :class_name => '::AccountsManagement::Accounts::BankAccount'

        def address_for_invoice
          return billing_address.format_for_invoice if billing_address
          return postal_address.format_for_invoice if postal_address
          nil
        end

        def vat_number_for_invoice
          vat_registration_number.nil? ? 'n/a' : vat_registration_number
        end
      end
    end

    # Initialisation

  end

end
